@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 from AppiumLibrary.locators import ElementFinder
+from appium.webdriver.common.appiumby import AppiumBy
 from .keywordgroup import KeywordGroup
 from robot.libraries.BuiltIn import BuiltIn
 import ast
@@ -42,7 +43,10 @@ class _ElementKeywords(KeywordGroup):
         self._element_find(locator, True, True).click()
 
     def click_button(self, index_or_name):
-        """ Click button """
+        """*DEPRECATED!!* in selenium v4, use `Click Element` keyword.
+        Click button
+
+        """
         _platform_class_dict = {'ios': 'UIAButton',
                                 'android': 'android.widget.Button'}
         if self._is_support_platform(_platform_class_dict):
@@ -61,6 +65,16 @@ class _ElementKeywords(KeywordGroup):
         """
         self._element_find_by_text(text,exact_match).click()
 
+    def input_text_into_current_element(self, text):
+        """Types the given `text` into currently selected text field.
+
+            Android only.
+        """
+        self._info("Typing text '%s' into current text field" % text)
+        driver = self._current_application()
+        driver.set_clipboard_text(text)
+        driver.press_keycode(50, 0x1000 | 0x2000)
+
     def input_text(self, locator, text):
         """Types the given `text` into text field identified by `locator`.
 
@@ -347,6 +361,45 @@ class _ElementKeywords(KeywordGroup):
         """
         return self._element_find(locator, True, True)
 
+    def scroll_element_into_view(self, locator):
+        """Scrolls an element from given ``locator`` into view.
+        Arguments:
+        - ``locator``: The locator to find requested element. Key attributes for
+                       arbitrary elements are ``id`` and ``name``. See `introduction` for
+                       details about locating elements.
+        Examples:
+        | Scroll Element Into View | css=div.class |
+        """
+        if isinstance(locator, WebElement):
+            element = locator
+        else:
+            self._info("Scrolling element '%s' into view." % locator)
+            element = self._element_find(locator, True, True)
+        script = 'arguments[0].scrollIntoView()'
+        # pylint: disable=no-member
+        self._current_application().execute_script(script, element)
+        return element
+
+    def get_webelement_in_webelement(self, element, locator):
+        """ 
+        Returns a single [http://selenium-python.readthedocs.io/api.html#module-selenium.webdriver.remote.webelement|WebElement] 
+        objects matching ``locator`` that is a child of argument element.
+
+        This is useful when your HTML doesn't properly have id or name elements on all elements.
+        So the user can find an element with a tag and then search that elmements children.
+        """
+        elements = None
+        if isstr(locator):
+            _locator = locator
+            elements = self._element_finder.find(element, _locator, None)
+            if len(elements) == 0:
+                raise ValueError("Element locator '" + locator + "' did not match any elements.")
+            if len(elements) == 0: 
+                return None
+            return elements[0]
+        elif isinstance(locator, WebElement):
+            return locator
+
     def get_webelements(self, locator):
         """Returns list of [http://selenium-python.readthedocs.io/api.html#module-selenium.webdriver.remote.webelement|WebElement] objects matching ``locator``.
 
@@ -406,6 +459,17 @@ class _ElementKeywords(KeywordGroup):
         self._info("Element '%s' size: %s " % (locator, element_size))
         return element_size
 
+    def get_element_rect(self, locator):
+        """Gets dimensions and coordinates of an element
+
+        Key attributes for arbitrary elements are `id` and `name`. See
+        `introduction` for details about locating elements.
+        """
+        element = self._element_find(locator, True, True)
+        element_rect = element.rect
+        self._info("Element '%s' rect: %s " % (locator, element_rect))
+        return element_rect
+
     def get_text(self, locator):
         """Get element text (for hybrid and mobile browser use `xpath` locator, others might cause problem)
 
@@ -481,10 +545,11 @@ class _ElementKeywords(KeywordGroup):
         else:
             return False
 
+    # TODO: Remove all locators methods from _element.py
     def _click_element_by_name(self, name):
         driver = self._current_application()
         try:
-            element = driver.find_element_by_name(name)
+            element = driver.find_element(by=AppiumBy.NAME, value=name)
         except Exception as e:
             raise e
 
@@ -493,9 +558,10 @@ class _ElementKeywords(KeywordGroup):
         except Exception as e:
             raise 'Cannot click the element with name "%s"' % name
 
+    # TODO: Remove all locators from _element.py
     def _find_elements_by_class_name(self, class_name):
         driver = self._current_application()
-        elements = driver.find_elements_by_class_name(class_name)
+        elements = driver.find_elements(by=AppiumBy.CLASS_NAME, value=class_name)
         return elements
 
     def _find_element_by_class_name(self, class_name, index_or_name):
